from . import types

space:       types.Renderer = ' '.join
concatenate: types.Renderer = ''.join
underscore:  types.Renderer = '_'.join
hyphen:      types.Renderer = '-'.join
period:      types.Renderer = '.'.join
slash:       types.Renderer = '/'.join
